require 'spec_helper'

describe TodosController do

   # This should return the minimal set of attributes required to create a valid
   # Todo. As you add validations to Todo, be sure to
   # update the return value of this method accordingly.
   def valid_attributes
     FactoryGirl.attributes_for(:todo)
   end

   let(:user) { FactoryGirl.create(:user) }

   before(:each) do
     @request.env["devise.mapping"] = Devise.mappings[:user]
     sign_in user
   end

   describe "GET index" do
     it "assigns all todos as @todos" do
       todo = user.todos.create! valid_attributes
       get :index, {}
       assigns(:todos).should eq([todo])
     end
   end

   describe "GET show" do
     it "assigns the requested todo as @todo" do
       todo = user.todos.create! valid_attributes
       get :show, {:id => todo.to_param}
       assigns(:todo).should eq(todo)
     end
   end

   describe "GET new" do
     it "assigns a new todo as @todo" do
       get :new, {}
       assigns(:todo).should be_a_new(Todo)
     end
   end

   describe "GET edit" do
     it "assigns the requested todo as @todo" do
       todo = user.todos.create! valid_attributes
       get :edit, {:id => todo.to_param}
       assigns(:todo).should eq(todo)
     end
   end

   describe "POST create" do
     describe "with valid params" do
       it "creates a new Todo" do
         expect {
           post :create, {:todo => valid_attributes}
         }.to change(Todo, :count).by(1)
       end

       it "assigns a newly created todo as @todo" do
         post :create, {:todo => valid_attributes}
         assigns(:todo).should be_a(Todo)
         assigns(:todo).should be_persisted
       end

       it "redirects to the created todo" do
         post :create, {:todo => valid_attributes}
         response.should redirect_to(Todo.last)
       end
     end

     describe "with invalid params" do
       it "assigns a newly created but unsaved todo as @todo" do
         # Trigger the behavior that occurs when invalid params are submitted
         Todo.any_instance.stub(:save).and_return(false)
         post :create, {:todo => {}}
         assigns(:todo).should be_a_new(Todo)
       end

       it "re-renders the 'new' template" do
         # Trigger the behavior that occurs when invalid params are submitted
         Todo.any_instance.stub(:save).and_return(false)
         post :create, {:todo => {}}
         response.should render_template("new")
       end
     end
   end

   describe "PUT update" do
     describe "with valid params" do
       it "updates the requested todo" do
         todo = Todo.create! valid_attributes
         # Assuming there are no other todos in the database, this
         # specifies that the Todo created on the previous line
         # receives the :update_attributes message with whatever params are
         # submitted in the request.
         Todo.any_instance.should_receive(:update_attributes).with({'these' => 'params'})
         put :update, {:id => todo.to_param, :todo => {'these' => 'params'}}
       end

       it "assigns the requested todo as @todo" do
         todo = Todo.create! valid_attributes
         put :update, {:id => todo.to_param, :todo => valid_attributes}
         assigns(:todo).should eq(todo)
       end

       it "redirects to the todo" do
         todo = Todo.create! valid_attributes
         put :update, {:id => todo.to_param, :todo => valid_attributes}
         response.should redirect_to(todo)
       end
     end

     describe "with invalid params" do
       it "assigns the todo as @todo" do
         todo = Todo.create! valid_attributes
         # Trigger the behavior that occurs when invalid params are submitted
         Todo.any_instance.stub(:save).and_return(false)
         put :update, {:id => todo.to_param, :todo => {}}
         assigns(:todo).should eq(todo)
       end

       it "re-renders the 'edit' template" do
         todo = Todo.create! valid_attributes
         # Trigger the behavior that occurs when invalid params are submitted
         Todo.any_instance.stub(:save).and_return(false)
         put :update, {:id => todo.to_param, :todo => {}}
         response.should render_template("edit")
       end
     end
   end

   describe "DELETE destroy" do
     it "destroys the requested todo" do
       todo = Todo.create! valid_attributes
       expect {
         delete :destroy, {:id => todo.to_param}
       }.to change(Todo, :count).by(-1)
     end

     it "redirects to the todos list" do
       todo = Todo.create! valid_attributes
       delete :destroy, {:id => todo.to_param}
       response.should redirect_to(todos_url)
     end
   end
end
